//******************************************************************************
//* snes_utils.inc
//* (c) 2016 qwertymodo
//* 
//* Some basic SNES utility macros.  Currently only supports standard Hi/LoROM.
//*
//* seek: Sets PC to the input address and moves the write pointer according to
//*       defined Hi/LoROM mapping.
//*
//* sseek: Sets PC to the SlowROM equivalent of the input address, regardless
//*        of FastROM definition and sets the write pointer according to
//*        defined HiROM/LoROM mapping.
//*
//* fseek: Sets PC to the FastROM equivalent of the input address if FastROM is
//*        defined, otherwise directly to the given address, and sets the write
//*        pointer according to defined Hi/LoROM mapping.
//*
//* org:   xkas compatibility macro.  Simple alias for seek.
//*
//* jmf:   Jump Long FastROM.
//*
//* jsf:   Jump to Subroutine Long FastROM.
//******************************************************************************

if !{defined _snes_utils_inc} {
define _snes_utils_inc()

if {defined fastrom} {
    define rombank($800000)
} else {
    define rombank($000000)
}


macro seek(variable offset) {
    if {defined hirom} {
        origin (offset) & $3FFFFF
        base offset
    } else {
        origin ((offset & $7F0000) >> 1) | (offset & $7FFF)
        base offset
    }
}


macro sseek(variable offset) {
    if {defined hirom} {
        origin (offset) & $3FFFFF
        base offset & !{rombank}
    } else {
        origin ((offset & $7F0000) >> 1) | (offset & $7FFF)
        base offset & !{rombank}
    }
}


macro fseek(variable offset) {
    if {defined hirom} {
        origin (offset) & $3FFFFF
        base offset | {rombank}
    } else {
        origin ((offset & $7F0000) >> 1) | (offset & $7FFF)
        base offset | {rombank}
    }
}


macro org(variable offset) {
    seek(offset)
}


macro jmf(variable offset) {
    jml offset | {rombank}
}


macro jsf(variable offset) {
    jsl offset | {rombank}
}


} // _snes_utils_inc