arch snes.cpu

define hirom()

include "snes_utils.inc"


// ===Interrupt Handlers===
seek($C00500)
nmi:
irq:
    php
    rep #$30
    pha
    phb
    phd
    phx
    phy
    sep #$20
    lda $4210

// Increment frame count
    ldx $00
    inx
    stx $00
    txa
    and #$03
    beq cgram_load
    cmp #$01
    beq tile_load_1
    cmp #$02
    beq tile_load_2
    brl tile_load_3

// Load CGRAM via DMA transfer
cgram_load:
    stz $2121   // Start at color 0
    ldx #$2200
    stx $4300   // DMA destination: CGDATA, byte increment
    ldx #palette// Palette data
    stx $4302   // Low DMA source address
    lda.b #palette >> 16
    sta $4304   // High DMA source address
    ldx #$0200
    stx $4305   // Transfer 512 bytes
    lda #$01
    sta $420B   // Start DMA transfer

// Load last 25% of tile data via DMA transfer
    lda #$80
    sta $2115   // VRAM single word transfer, word increment
    ldx #$4600
    stx $2116   // VRAM address $2800
    ldx #$1801
    stx $4300   // DMA destination: VMDATAL/VMDATAH, word increment
    ldx #tiles+$6C00  // Tile data
    stx $4302   // Low DMA source address
    lda.b #tiles >> 16
    sta $4304   // High DMA source address
    ldx #$2440
    stx $4305   // Transfer 9,280 (1/4 of the tiles)
    lda #$01
    sta $420B   // Start DMA transfer
    bra tile_load_done

// Load first 25% of tile data via DMA transfer
tile_load_1:
    lda #$80
    sta $2115   // VRAM single word transfer, word increment
    ldx #$1000
    stx $2116   // VRAM address $1000
    ldx #$1801
    stx $4300   // DMA destination: VMDATAL/VMDATAH, word increment
    ldx #tiles  // Tile data
    stx $4302   // Low DMA source address
    lda.b #tiles >> 16
    sta $4304   // High DMA source address
    ldx #$2400
    stx $4305   // Transfer 9,216 (1/4 of the tiles)
    lda #$01
    sta $420B   // Start DMA transfer
    bra tile_load_done

// Load second 25% of tile data via DMA transfer
tile_load_2:
    lda #$80
    sta $2115   // VRAM single word transfer, word increment
    ldx #$2200
    stx $2116   // VRAM address $1800
    ldx #$1801
    stx $4300   // DMA destination: VMDATAL/VMDATAH, word increment
    ldx #tiles+$2400  // Tile data
    stx $4302   // Low DMA source address
    lda.b #tiles >> 16
    sta $4304   // High DMA source address
    ldx #$2400
    stx $4305   // Transfer 9,216 (1/4 of the tiles)
    lda #$01
    sta $420B   // Start DMA transfer
    bra tile_load_done

// Load third 25% of tile data via DMA transfer
tile_load_3:
    lda #$80
    sta $2115   // VRAM single word transfer, word increment
    ldx #$3400
    stx $2116   // VRAM address $1000 (word address $0800)
    ldx #$1801
    stx $4300   // DMA destination: VMDATAL/VMDATAH, word increment
    ldx #tiles+$4800  // Tile data
    stx $4302   // Low DMA source address
    lda.b #tiles >> 16
    sta $4304   // High DMA source address
    ldx #$2400
    stx $4305   // Transfer 9,216 (1/4 of the tiles)
    lda #$01
    sta $420B   // Start DMA transfer

tile_load_done:

    rep #$30
    ply
    plx
    pld
    plb
    pla
    plp
    rti

// ===Initialization Routine===
seek($C02000)
__init:
    sep #$30    // X,Y,A are 8 bit numbers
    lda #$8F    // screen off, full brightness
    sta $2100   // brightness + screen enable register 
    stz $2101   // Sprite register (size + address in VRAM) 
    stz $2102   // Sprite registers (address of sprite memory [OAM])
    stz $2103   //    ""                       ""
    stz $2105   // Mode 0, = Graphic mode register
    stz $2106   // noplanes, no mosaic, = Mosaic register
    stz $2107   // Plane 0 map VRAM location
    stz $2108   // Plane 1 map VRAM location
    stz $2109   // Plane 2 map VRAM location
    stz $210A   // Plane 3 map VRAM location
    stz $210B   // Plane 0+1 Tile data location
    stz $210C   // Plane 2+3 Tile data location
    stz $210D   // Plane 0 scroll x (first 8 bits)
    stz $210D   // Plane 0 scroll x (last 3 bits) #$0 - #$07ff
    lda #$FF    // The top pixel drawn on the screen isn't the top one in the tilemap, it's the one above that.
    sta $210E
    sta $210E   // Plane 0 scroll y
    sta $2110
    sta $2110   // Plane 1 scroll y
    sta $2112
    sta $2112   // Plane 2 scroll y
    sta $2114
    sta $2114   // Plane 3 scroll y
    stz $210F   // Plane 1 scroll x (first 8 bits)
    stz $210F   // Plane 1 scroll x (last 3 bits) #$0 - #$07ff
    stz $2111   // Plane 2 scroll x (first 8 bits)
    stz $2111   // Plane 2 scroll x (last 3 bits) #$0 - #$07ff
    stz $2113   // Plane 3 scroll x (first 8 bits)
    stz $2113   // Plane 3 scroll x (last 3 bits) #$0 - #$07ff
    lda #$80    // increase VRAM address after writing to $2119
    sta $2115   // VRAM address increment register
    stz $2116   // VRAM address low
    stz $2117   // VRAM address high
    stz $211A   // Initial Mode 7 setting register
    stz $211B   // Mode 7 matrix parameter A register (low)
    lda #$01
    sta $211B   // Mode 7 matrix parameter A register (high)
    stz $211C   // Mode 7 matrix parameter B register (low)
    stz $211C   // Mode 7 matrix parameter B register (high)
    stz $211D   // Mode 7 matrix parameter C register (low)
    stz $211D   // Mode 7 matrix parameter C register (high)
    stz $211E   // Mode 7 matrix parameter D register (low)
    sta $211E   // Mode 7 matrix parameter D register (high)
    stz $211F   // Mode 7 center position X register (low)
    stz $211F   // Mode 7 center position X register (high)
    stz $2120   // Mode 7 center position Y register (low)
    stz $2120   // Mode 7 center position Y register (high)
    stz $2121   // Color number register ($0-ff)
    stz $2123   // BG1 & BG2 Window mask setting register
    stz $2124   // BG3 & BG4 Window mask setting register
    stz $2125   // OBJ & Color Window mask setting register
    stz $2126   // Window 1 left position register
    stz $2127   // Window 2 left position register
    stz $2128   // Window 3 left position register
    stz $2129   // Window 4 left position register
    stz $212A   // BG1, BG2, BG3, BG4 Window Logic register
    stz $212B   // OBJ, Color Window Logic Register (or,and,xor,xnor)
    sta $212C   // Main Screen designation (planes, sprites enable)
    stz $212D   // Sub Screen designation
    stz $212E   // Window mask for Main Screen
    stz $212F   // Window mask for Sub Screen
    lda #$30
    sta $2130   // Color addition & screen addition init setting
    stz $2131   // Add/Sub sub designation for screen, sprite, color
    lda #$E0
    sta $2132   // color data for addition/subtraction
    stz $2133   // Screen setting (interlace x,y/enable SFX data)
    stz $4200   // Disable V-blank, interrupt, Joypad register
    lda #$FF
    sta $4201   // Programmable I/O port
    stz $4202   // Multiplicand A
    stz $4203   // Multiplier B
    stz $4204   // Multiplier C
    stz $4205   // Multiplicand C
    stz $4206   // Divisor B
    stz $4207   // Horizontal Count Timer
    stz $4208   // Horizontal Count Timer MSB (most significant bit)
    stz $4209   // Vertical Count Timer
    stz $420A   // Vertical Count Timer MSB
    stz $420B   // General DMA enable (bits 0-7)
    stz $420C   // Horizontal DMA (HDMA) enable (bits 0-7)
    stz $420D   // Access cycle designation (slow/fast rom)
    rep #$10    // Switch to 16-bit X/Y
    stz $2121   // Clear CGRAM via DMA transfer
    ldx #$2208
    stx $4300   // DMA destination: CGDATA, fixed address
    ldx snes_header+7   // Convenient $0000 word in ROM
    stx $4302   // Low DMA source address
    lda snes_header+7 >> 16
    sta $4304   // High DMA source address
    ldx #$0200
    stx $4305   // Transfer 512 bytes
    lda #$01
    sta $420B   // Start DMA transfer
    ldx #$0000  // Clear VRAM via DMA transfer
    stx $2116
    ldx #$1809
    stx $4300   // DMA destination: VMDATAL/VMDATAH, fixed address
    ldx snes_header+7   // Convenient $0000 word in ROM
    stx $4302   // Low DMA source address
    lda snes_header+7 >> 16
    sta $4304   // High DMA source address
    ldx #$0000
    stx $4305   // Transfer 65536 bytes
    lda #$01
    sta $420B   // Start DMA transfer

// Load tilemap data
    lda #$80
    sta $2115   // VRAM single word transfer, word increment
    ldx #$0000
    stx $2116   // VRAM address $0000
    ldx #$1801
    stx $4300   // DMA destination: VMDATAL/VMDATAH, word increment
    ldx #tilemap// Tile data
    stx $4302   // Low DMA source address
    lda.b #tilemap >> 16
    sta $4304   // High DMA source address
    ldx #$0800
    stx $4305   // Transfer 2048 bytes
    lda #$01
    sta $420B   // Start DMA transfer

    sep #$20    // Switch to 8-bit x,y
    cli         // Enable interrupts
    jml __main

// ===Interrupt Vectors===
seek($C0FF00)
// RESET
    sei
    clc
    xce
    rep #$18
    ldx #$1FFF
    txs
    jml __init

seek($C0FF10)
// NMI
    jml nmi

seek($C0FF14)
// IRQ
    jml irq

seek($C0FF18)
// BRK
-//  lda $ABCDEF
    bra -

seek($C0FFFF)
// EMPTY VECTOR
    rti

seek($C0FFB0)
// ===Internal Header===
snes_header:
    dw $3713            // Maker code
    db 'M','S','U','1'  // Game code
    // Reserved
    db $00,$00,$00,$00,$00,$00,$00
    db $00              // Exp RAM size
    db $00              // Special version
    db $00              // Cartridge type
    // ROM Name
    db 'M','S','U','-','1',' ','V','i','d','e','o'
    db ' ','T','e','s','t',' ','R','O','M',' '
    db $31              // Mapper
    db $02              // ROM type
    db $0C              // ROM size
    db $03              // SRAM size
    db $01              // Country code
    db $33              // Reserved
    db $01              // Version number
    dw $AA00            // Checksum complement
    dw $55FF            // Checksum

// ===Vector Table===
// Native Mode
    dd $FFFFFFFF        // UNUSED
    dw $FFFF            // COP
    dw $FF18            // BRK
    dw $FFFF            // ABORT
    dw $FF10            // NMI
    dw $FFFF            // RESET
    dw $FF14            // IRQ

// Emulation Mode
    dd $FFFFFFFF        // UNUSED
    dw $FF18            // COP
    dw $FFFF            // BRK
    dw $FFFF            // ABORT
    dw $FFFF            // NMI
    dw $FF00            // RESET
    dw $FFFF            // IRQ


seek($D00000)
__main:
    sep #$20        // Set the A register to 8-bit.

// Setup video mode
    lda #$03
    sta $2105   // Set video mode 3, 8x8 tiles, 256 color BG1, 16 color BG2

    lda #$00
    sta $2107   // Set BG1 tilemap offset to $0000 and size to 32x32
    
    lda #$01
    sta $210B   // Set BG1 tile data offset to $1000

    lda #$01
    sta $212C   // Enable BG1

    lda #$FF
    sta $210E   // Set BG1 scroll register
    sta $210E
    
    lda #$0F
    sta $2100   // Turn on screen, full brightness

    stz $00     // Initialize frame counter
    stz $01

    lda #$80
    sta $4200   // Enable NMI

// Loop forever.
-;  //wai
    bra -


seek($E00000)
include "cgram.inc"
include "tiles.inc"
include "tilemap.inc"

seek($FFFFFF)
__end:
    db $00